#include <stdio.h>
#include <string.h>
#include "externs.h"
#include "db.h"

#define CA_BRIGHT	1
#define CA_REVERSE	2
#define CA_UNDERLINE	4

static int color2num(char);
static int is_foreground(int);
static int is_background(int);
static char *make_num_string(int, int, int);
static void set_ca(int *, int);
static char *color_escape(const char *);
static char *normal = "\e[0m";
static char *colorize(const char *, int);
static char *strip_beep(char *);

static char *colorize(str, strip)
     const char *str;
     int strip;
{
  char *s, *colorend, *colorstr, *escape;
  static char buf[2048];

  strcpy(buf,str);

  for(s = buf; *s; s++) {
    if (*s == '|') {
      colorstr = strpbrk(s+1,"+|");	/* Look and see if it's color.
                                           If it is, colorstr points to
                                           the plus before the string
                                           to be colorized. */
      if (colorstr && *colorstr == '|') /* If it's not, keep going. */
        continue;
      else if (colorstr) {
        if(*(colorstr+1) == '{') { /* If the colorstr begins and ends with */
          char *end_curl;
          end_curl = strchr(colorstr+1,'}');  /* curlies, ignore any bars  */
          if(end_curl && *(end_curl+1) == '|') {          /* within. */
            *end_curl++ = '\0';
            *colorstr++ = '\0';
            colorend = end_curl;
          } else
            colorend = strchr(colorstr+1,'|');
        } else
          colorend = strchr(colorstr+1,'|'); /* Find the end of the color. */
        if (colorend && *colorend) {       /* If it has an end, parse it. */
          char buf2[2048];

          *s++ = '\0';
          *colorstr++ = '\0';
          *colorend++ = '\0';

          strcpy(buf2,buf);

          if (!strip) {
            escape = color_escape(s);
            s += strlen(escape) + strlen(colorstr) + strlen(normal) - 2;
            strcat(buf2,escape);
            strcat(buf2,colorstr);
            strcat(buf2,normal);
          } else {
            s += strlen(colorstr) - 2;
            strcat(buf2,colorstr);
          }

          strcat(buf2,colorend);
          strcpy(buf,buf2);
        }
      }
    }
  }
  return buf;
}

static int color2num (c)
     char c;
{
  switch(c) {
    case '!':
      return 1;
      break;
    case 'u':
      return 4;
      break;
 /* case 'b':
      return 5;
      break;  no blinking! */
    case 'r':
      return 7;
      break;
    case 'N':
      return 30;
      break;
    case 'R':
      return 31;
      break;
    case 'G':
      return 32;
      break;
    case 'Y':
      return 33;
      break;
    case 'B':
      return 34;
      break;
    case 'M':
      return 35;
      break;
    case 'C':
      return 36;
      break;
    case 'W':
      return 37;
      break;
    case '0':
      return 40;
      break;
    case '1':
      return 41;
      break;
    case '2':
      return 42;
      break;
    case '3':
      return 43;
      break;
    case '4':
      return 44;
      break;
    case '5':
      return 45;
      break;
    case '6':
      return 46;
      break;
    case '7':
      return 47;
      break;
    default:
      return -1;
  }
}

static int is_foreground(num)
     int num;
{
  return (num >= 30 && num <= 37);
}

static int is_background(num)
     int num;
{
  return (num >= 40 && num <= 47);
}

static char *make_num_string(fore, back, ca)
     int fore, back, ca;
{
  static char buff[2048];

  strcpy(buff,"");

  if (fore > 0)
    sprintf(buff, "%s%d;", buff, fore);

  if (back > 0)
    sprintf(buff, "%s%d;", buff, back);

  if (ca > 0)
    if (ca & CA_BRIGHT)
      strcat(buff, "1;");
    if (ca & CA_REVERSE)
      strcat(buff, "7;");
    if (ca & CA_UNDERLINE)
      strcat(buff, "4;");

  *strrchr(buff, ';') = '\0';

  return buff;
}

static void set_ca(attribs, num)
     int *attribs, num;
{
  if (num == 1)
    *attribs |= CA_BRIGHT;
  else if (num == 7)
    *attribs |= CA_REVERSE;
  else if (num == 4)
    *attribs |= CA_UNDERLINE;
  else
    log_error("Eeek! Something evil happened in set_ca!");
}

static char *color_escape(s)
     const char *s;
{
  static char escape[2048];
  int foreground = 37;
  int background = 40;
  int attribs = 0;
  int num, valid;

  for(valid = 0; *s; s++) {
    num = color2num(*s);
    if (num > 0) {
      valid = 1;
      if (is_foreground(num))
        foreground = num;
      else if (is_background(num))
        background = num;
      else
        set_ca(&attribs,num);
    }
  }

  if (valid) {
    if (foreground == (background-10))
      if (foreground == 30 && !(attribs&CA_BRIGHT))
        background = 47;
      else
        background = 40;

    sprintf(escape, "\e[%sm",make_num_string(foreground, background, attribs));
  } else
    strcpy(escape,"");

  return escape;
}

char *strip_color(str)
     const char *str;
{
  return colorize(str,1);
}

char *strip_color_nobeep(str)
     char *str;
{
  return colorize(strip_beep(str),1);
}

char *parse_color(str)
     const char *str;
{
  return colorize(str,0);
}

char *parse_color_nobeep(str)
     char *str;
{
  return colorize(strip_beep(str),0);
}

static char *strip_beep(str)
     char *str;
{
  static char buf[2048];
  char *s, *p;

  for( p=buf, s=str; *s; s++)
    if (*s != '\a')
      *p++ = *s;

  *p = '\0';

  return buf;
}

char *truncate_color(str, num)
     char *str;
     int num;
{
  static char buf[2048];
  char *s;
  int count = 0;

  strcpy(buf,str);

  for(s = buf; *s && count < num; s++) {
    if( *s == '|' ) {
      char *color;
      color = strchr(s,'+');
      if (color && *(color+1) == '{') {
        char *end_curl;
        end_curl = strchr(color+2,'}');
        if (end_curl && *(end_curl+1) == '|') {
          for(s = color+2; *s != '}' && count < num; s++, count++);
          if (count >= num) {
            *s++ = '}';
            *s = '|';
          } else
            s++;
        } else if (strchr(color+1,'|')) {
          for(s = color+1; *s != '|' && count < num; s++, count++);
          if (count >= num)
            *s = '|';
        }
      } else if (color && strchr(color+1,'|')) {
        for(s = color+1; *s != '|' && count < num; s++, count++);
        if (count >= num)
          *s = '|';
      }
    } else
      count++;
  }

  *s = '\0';
  
  return buf;
}
